#####################################################################################################
# CROWDSTRIKE FALCON                                                                                #
# oAuth2 API - Customer SDK                                                                         #
#                                                                                                   #
# api_complete.py - All-in-one CrowdStrike Falcon oAuth2 API harness                                #
#####################################################################################################
# Copyright CrowdStrike 2020

# By accessing or using this script, sample code, application programming interface, tools, 
# and/or associated documentation (if any) (collectively, “Tools”), You (i) represent and 
# warrant that You are entering into this Agreement on behalf of a company, organization 
# or another legal entity (“Entity”) that is currently a customer or partner of 
# CrowdStrike, Inc. (“CrowdStrike”), and (ii) have the authority to bind such Entity and 
# such Entity agrees to be bound by this Agreement.

# CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty 
# free and limited license to access and use the Tools solely for Entity’s internal business 
# purposes and in accordance with its obligations under any agreement(s) it may have with 
# CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its licensors retain all 
# right, title and interest in and to the Tools, and all intellectual property rights 
# embodied therein, and that Entity has no right, title or interest therein except for the 
# express licenses granted hereunder and that Entity will treat such Tools as CrowdStrike’s 
# confidential information.

# THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR 
# STATUTORY OR OTHERWISE. CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND 
# ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL CROWDSTRIKE 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json
import time
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

class APIHarness:
    """ This one does it all. It's like the One Ring with significantly fewer orcs. """

    def __init__(self, creds, base_url="https://api.crowdstrike.com"):
        """ Instantiates an instance of the base class, ingests credentials and the base URL and initializes global variables. """
    
        self.creds = creds
        self.base_url = base_url
        self.token = False
        self.token_expiration = 0
        self.token_renew_window = 20
        self.token_time = time.time()
        self.token_expired = lambda: True if (time.time() - self.token_time) >= (self.token_expiration - self.token_renew_window) else False
        self.authenticated = False
        self.valid_cred_format = lambda: True if "client_id" in self.creds and "client_secret" in self.creds else False
        self.headers = lambda: { 'Authorization': 'Bearer {}'.format(self.token) } if self.token else {}
        # This is a list of available commands, additional endpoints (commands) can be added ad-hoc
        # [Command Name, HTTP Method, Endpoint]
        self.commands = [
            ["QueryAWSAccounts", "GET", "/cloud-connect-aws/combined/accounts/v1"], 
            ["GetAWSSettings", "GET", "/cloud-connect-aws/combined/settings/v1"], 
            ["GetAWSAccounts", "GET", "/cloud-connect-aws/entities/accounts/v1?ids={}"], 
            ["ProvisionAWSAccounts", "POST", "/cloud-connect-aws/entities/accounts/v1"], 
            ["UpdateAWSAccounts", "PATCH", "/cloud-connect-aws/entities/accounts/v1"], 
            ["DeleteAWSAccounts", "DELETE", "/cloud-connect-aws/entities/accounts/v1?ids={}"], 
            ["CreateOrUpdateAWSSettings", "POST", "/cloud-connect-aws/entities/settings/v1"], 
            ["VerifyAWSAccountAccess", "POST", "/cloud-connect-aws/entities/verify-account-access/v1?ids={}"], 
            ["QueryAWSAccountsForIDs", "GET", "/cloud-connect-aws/queries/accounts/v1"], 
            ["GetCSPMAzureAccount", "GET", "/cloud-connect-azure/entities/account/v1?ids={}"], 
            ["CreateCSPMAzureAccount", "POST", "/cloud-connect-azure/entities/account/v1"], 
            ["UpdateCSPMAzureAccountClientID", "PATCH", "/cloud-connect-azure/entities/client-id/v1"], 
            ["GetCSPMAzureUserScriptsAttachment", "GET", "/cloud-connect-azure/entities/user-scripts-download/v1"], 
            ["GetCSPMAzureUserScripts", "GET", "/cloud-connect-azure/entities/user-scripts/v1"], 
            ["GetCSPMAwsAccount", "GET", "/cloud-connect-cspm-aws/entities/account/v1?ids={}"], 
            ["CreateCSPMAwsAccount", "POST", "/cloud-connect-cspm-aws/entities/account/v1"], 
            ["DeleteCSPMAwsAccount", "DELETE", "/cloud-connect-cspm-aws/entities/account/v1?ids={}"], 
            ["GetCSPMAwsConsoleSetupURLs", "GET", "/cloud-connect-cspm-aws/entities/console-setup-urls/v1"], 
            ["GetCSPMAwsAccountScriptsAttachment", "GET", "/cloud-connect-cspm-aws/entities/user-scripts-download/v1"], 
            ["GetCSPMAzureAccount", "GET", "/cloud-connect-cspm-azure/entities/account/v1?ids={}"], 
            ["CreateCSPMAzureAccount", "POST", "/cloud-connect-cspm-azure/entities/account/v1"], 
            ["DeleteCSPMAzureAccount", "DELETE", "/cloud-connect-cspm-azure/entities/account/v1?ids={}"], 
            ["UpdateCSPMAzureAccountClientID", "PATCH", "/cloud-connect-cspm-azure/entities/client-id/v1"], 
            ["GetCSPMAzureUserScriptsAttachment", "GET", "/cloud-connect-cspm-azure/entities/user-scripts-download/v1"], 
            ["GetCSPMCGPAccount", "GET", "/cloud-connect-gcp/entities/account/v1?ids={}"], 
            ["CreateCSPMGCPAccount", "POST", "/cloud-connect-gcp/entities/account/v1"], 
            ["GetCSPMGCPUserScriptsAttachment", "GET", "/cloud-connect-gcp/entities/user-scripts-download/v1"], 
            ["GetCSPMGCPUserScripts", "GET", "/cloud-connect-gcp/entities/user-scripts/v1"], 
            ["GetAggregateDetects", "POST", "/detects/aggregates/detects/GET/v1"], 
            ["UpdateDetectsByIdsV2", "PATCH", "/detects/entities/detects/v2"], 
            ["GetDetectSummaries", "POST", "/detects/entities/summaries/GET/v1"], 
            ["QueryDetects", "GET", "/detects/queries/detects/v1"], 
            ["queryCombinedGroupMembers", "GET", "/devices/combined/host-group-members/v1"], 
            ["queryCombinedHostGroups", "GET", "/devices/combined/host-groups/v1"], 
            ["PerformActionV2", "POST", "/devices/entities/devices-actions/v2"], 
            ["GetDeviceDetails", "GET", "/devices/entities/devices/v1?ids={}"], 
            ["performGroupAction", "POST", "/devices/entities/host-group-actions/v1"], 
            ["getHostGroups", "GET", "/devices/entities/host-groups/v1?ids={}"], 
            ["createHostGroups", "POST", "/devices/entities/host-groups/v1"], 
            ["updateHostGroups", "PATCH", "/devices/entities/host-groups/v1"], 
            ["deleteHostGroups", "DELETE", "/devices/entities/host-groups/v1?ids={}"], 
            ["QueryHiddenDevices", "GET", "/devices/queries/devices-hidden/v1"], 
            ["QueryDevicesByFilterScroll", "GET", "/devices/queries/devices-scroll/v1"], 
            ["QueryDevicesByFilter", "GET", "/devices/queries/devices/v1"], 
            ["queryGroupMembers", "GET", "/devices/queries/host-group-members/v1"], 
            ["queryHostGroups", "GET", "/devices/queries/host-groups/v1"], 
            ["GetArtifacts", "GET", "/falconx/entities/artifacts/v1"], 
            ["GetSummaryReports", "GET", "/falconx/entities/report-summaries/v1?ids={}"], 
            ["GetReports", "GET", "/falconx/entities/reports/v1?ids={}"], 
            ["DeleteReport", "DELETE", "/falconx/entities/reports/v1?ids={}"], 
            ["GetSubmissions", "GET", "/falconx/entities/submissions/v1?ids={}"], 
            ["Submit", "POST", "/falconx/entities/submissions/v1"], 
            ["QueryReports", "GET", "/falconx/queries/reports/v1"], 
            ["QuerySubmissions", "GET", "/falconx/queries/submissions/v1"], 
            ["aggregate-events", "POST", "/fwmgr/aggregates/events/GET/v1"], 
            ["aggregate-policy-rules", "POST", "/fwmgr/aggregates/policy-rules/GET/v1"], 
            ["aggregate-rule-groups", "POST", "/fwmgr/aggregates/rule-groups/GET/v1"], 
            ["aggregate-rules", "POST", "/fwmgr/aggregates/rules/GET/v1"], 
            ["get-events", "GET", "/fwmgr/entities/events/v1?ids={}"], 
            ["get-firewall-fields", "GET", "/fwmgr/entities/firewall-fields/v1?ids={}"], 
            ["get-platforms", "GET", "/fwmgr/entities/platforms/v1?ids={}"], 
            ["get-policy-containers", "GET", "/fwmgr/entities/policies/v1?ids={}"], 
            ["get-rule-groups", "GET", "/fwmgr/entities/rule-groups/v1?ids={}"], 
            ["create-rule-group", "POST", "/fwmgr/entities/rule-groups/v1"], 
            ["update-rule-group", "PATCH", "/fwmgr/entities/rule-groups/v1"], 
            ["delete-rule-groups", "DELETE", "/fwmgr/entities/rule-groups/v1?ids={}"], 
            ["get-rules", "GET", "/fwmgr/entities/rules/v1?ids={}"], 
            ["query-events", "GET", "/fwmgr/queries/events/v1"], 
            ["query-firewall-fields", "GET", "/fwmgr/queries/firewall-fields/v1"], 
            ["query-platforms", "GET", "/fwmgr/queries/platforms/v1"], 
            ["query-policy-rules", "GET", "/fwmgr/queries/policy-rules/v1"], 
            ["query-rule-groups", "GET", "/fwmgr/queries/rule-groups/v1"], 
            ["query-rules", "GET", "/fwmgr/queries/rules/v1"], 
            ["CrowdScore", "GET", "/incidents/combined/crowdscores/v1"], 
            ["GetBehaviors", "POST", "/incidents/entities/behaviors/GET/v1"], 
            ["PerformIncidentAction", "POST", "/incidents/entities/incident-actions/v1"], 
            ["GetIncidents", "POST", "/incidents/entities/incidents/GET/v1"], 
            ["QueryBehaviors", "GET", "/incidents/queries/behaviors/v1"], 
            ["QueryIncidents", "GET", "/incidents/queries/incidents/v1"], 
            ["DevicesCount", "GET", "/indicators/aggregates/devices-count/v1"], 
            ["GetIOC", "GET", "/indicators/entities/iocs/v1"], 
            ["CreateIOC", "POST", "/indicators/entities/iocs/v1"], 
            ["UpdateIOC", "PATCH", "/indicators/entities/iocs/v1"], 
            ["DeleteIOC", "DELETE", "/indicators/entities/iocs/v1"], 
            ["DevicesRanOn", "GET", "/indicators/queries/devices/v1"], 
            ["QueryIOCs", "GET", "/indicators/queries/iocs/v1"], 
            ["ProcessesRanOn", "GET", "/indicators/queries/processes/v1"], 
            ["audit-events-read", "GET", "/installation-tokens/entities/audit-events/v1?ids={}"], 
            ["customer-settings-read", "GET", "/installation-tokens/entities/customer-settings/v1"], 
            ["tokens-read", "GET", "/installation-tokens/entities/tokens/v1?ids={}"], 
            ["tokens-create", "POST", "/installation-tokens/entities/tokens/v1"], 
            ["tokens-update", "PATCH", "/installation-tokens/entities/tokens/v1?ids={}"], 
            ["tokens-delete", "DELETE", "/installation-tokens/entities/tokens/v1?ids={}"], 
            ["audit-events-query", "GET", "/installation-tokens/queries/audit-events/v1"], 
            ["tokens-query", "GET", "/installation-tokens/queries/tokens/v1"], 
            ["QueryIntelActorEntities", "GET", "/intel/combined/actors/v1"], 
            ["QueryIntelIndicatorEntities", "GET", "/intel/combined/indicators/v1"], 
            ["QueryIntelReportEntities", "GET", "/intel/combined/reports/v1"], 
            ["GetIntelActorEntities", "GET", "/intel/entities/actors/v1?ids={}"], 
            ["GetIntelIndicatorEntities", "POST", "/intel/entities/indicators/GET/v1"], 
            ["GetIntelReportPDF", "GET", "/intel/entities/report-files/v1"], 
            ["GetIntelReportEntities", "GET", "/intel/entities/reports/v1?ids={}"], 
            ["GetIntelRuleFile", "GET", "/intel/entities/rules-files/v1"], 
            ["GetLatestIntelRuleFile", "GET", "/intel/entities/rules-latest-files/v1"], 
            ["GetIntelRuleEntities", "GET", "/intel/entities/rules/v1?ids={}"], 
            ["QueryIntelActorIds", "GET", "/intel/queries/actors/v1"], 
            ["QueryIntelIndicatorIds", "GET", "/intel/queries/indicators/v1"], 
            ["QueryIntelReportIds", "GET", "/intel/queries/reports/v1"], 
            ["QueryIntelRuleIds", "GET", "/intel/queries/rules/v1"], 
            ["get-patterns", "GET", "/ioarules/entities/pattern-severities/v1?ids={}"], 
            ["get-platformsMixin0", "GET", "/ioarules/entities/platforms/v1?ids={}"], 
            ["get-rule-groupsMixin0", "GET", "/ioarules/entities/rule-groups/v1?ids={}"], 
            ["create-rule-groupMixin0", "POST", "/ioarules/entities/rule-groups/v1"], 
            ["update-rule-groupMixin0", "PATCH", "/ioarules/entities/rule-groups/v1"], 
            ["delete-rule-groupsMixin0", "DELETE", "/ioarules/entities/rule-groups/v1?ids={}"], 
            ["get-rule-types", "GET", "/ioarules/entities/rule-types/v1?ids={}"], 
            ["get-rules-get", "POST", "/ioarules/entities/rules/GET/v1"], 
            ["get-rulesMixin0", "GET", "/ioarules/entities/rules/v1?ids={}"], 
            ["create-rule", "POST", "/ioarules/entities/rules/v1"], 
            ["update-rules", "PATCH", "/ioarules/entities/rules/v1"], 
            ["delete-rules", "DELETE", "/ioarules/entities/rules/v1?ids={}"], 
            ["validate", "POST", "/ioarules/entities/rules/validate/v1"], 
            ["query-patterns", "GET", "/ioarules/queries/pattern-severities/v1"], 
            ["query-platformsMixin0", "GET", "/ioarules/queries/platforms/v1"], 
            ["query-rule-groups-full", "GET", "/ioarules/queries/rule-groups-full/v1"], 
            ["query-rule-groupsMixin0", "GET", "/ioarules/queries/rule-groups/v1"], 
            ["query-rule-types", "GET", "/ioarules/queries/rule-types/v1"], 
            ["query-rulesMixin0", "GET", "/ioarules/queries/rules/v1"], 
            ["GetMalQueryQuotasV1", "GET", "/malquery/aggregates/quotas/v1"], 
            ["PostMalQueryFuzzySearchV1", "POST", "/malquery/combined/fuzzy-search/v1"], 
            ["GetMalQueryDownloadV1", "GET", "/malquery/entities/download-files/v1?ids={}"], 
            ["GetMalQueryMetadataV1", "GET", "/malquery/entities/metadata/v1?ids={}"], 
            ["GetMalQueryRequestV1", "GET", "/malquery/entities/requests/v1?ids={}"], 
            ["GetMalQueryEntitiesSamplesFetchV1", "GET", "/malquery/entities/samples-fetch/v1?ids={}"], 
            ["PostMalQueryEntitiesSamplesMultidownloadV1", "POST", "/malquery/entities/samples-multidownload/v1"], 
            ["PostMalQueryExactSearchV1", "POST", "/malquery/queries/exact-search/v1"], 
            ["PostMalQueryHuntV1", "POST", "/malquery/queries/hunt/v1"], 
            ["oauth2RevokeToken", "POST", "/oauth2/revoke"], 
            ["oauth2AccessToken", "POST", "/oauth2/token"], 
            ["queryCombinedDeviceControlPolicyMembers", "GET", "/policy/combined/device-control-members/v1"], 
            ["queryCombinedDeviceControlPolicies", "GET", "/policy/combined/device-control/v1"], 
            ["queryCombinedFirewallPolicyMembers", "GET", "/policy/combined/firewall-members/v1"], 
            ["queryCombinedFirewallPolicies", "GET", "/policy/combined/firewall/v1"], 
            ["queryCombinedPreventionPolicyMembers", "GET", "/policy/combined/prevention-members/v1"], 
            ["queryCombinedPreventionPolicies", "GET", "/policy/combined/prevention/v1"], 
            ["revealUninstallToken", "POST", "/policy/combined/reveal-uninstall-token/v1"], 
            ["queryCombinedSensorUpdateBuilds", "GET", "/policy/combined/sensor-update-builds/v1"], 
            ["queryCombinedSensorUpdatePolicyMembers", "GET", "/policy/combined/sensor-update-members/v1"], 
            ["queryCombinedSensorUpdatePolicies", "GET", "/policy/combined/sensor-update/v1"], 
            ["queryCombinedSensorUpdatePoliciesV2", "GET", "/policy/combined/sensor-update/v2"], 
            ["performDeviceControlPoliciesAction", "POST", "/policy/entities/device-control-actions/v1"], 
            ["setDeviceControlPoliciesPrecedence", "POST", "/policy/entities/device-control-precedence/v1"], 
            ["getDeviceControlPolicies", "GET", "/policy/entities/device-control/v1?ids={}"], 
            ["createDeviceControlPolicies", "POST", "/policy/entities/device-control/v1"], 
            ["updateDeviceControlPolicies", "PATCH", "/policy/entities/device-control/v1"], 
            ["deleteDeviceControlPolicies", "DELETE", "/policy/entities/device-control/v1?ids={}"], 
            ["performFirewallPoliciesAction", "POST", "/policy/entities/firewall-actions/v1"], 
            ["setFirewallPoliciesPrecedence", "POST", "/policy/entities/firewall-precedence/v1"], 
            ["getFirewallPolicies", "GET", "/policy/entities/firewall/v1?ids={}"], 
            ["createFirewallPolicies", "POST", "/policy/entities/firewall/v1"], 
            ["updateFirewallPolicies", "PATCH", "/policy/entities/firewall/v1"], 
            ["deleteFirewallPolicies", "DELETE", "/policy/entities/firewall/v1?ids={}"], 
            ["getIOAExclusionsV1", "GET", "/policy/entities/ioa-exclusions/v1?ids={}"], 
            ["createIOAExclusionsV1", "POST", "/policy/entities/ioa-exclusions/v1"], 
            ["updateIOAExclusionsV1", "PATCH", "/policy/entities/ioa-exclusions/v1"], 
            ["deleteIOAExclusionsV1", "DELETE", "/policy/entities/ioa-exclusions/v1?ids={}"], 
            ["getMLExclusionsV1", "GET", "/policy/entities/ml-exclusions/v1?ids={}"], 
            ["createMLExclusionsV1", "POST", "/policy/entities/ml-exclusions/v1"], 
            ["updateMLExclusionsV1", "PATCH", "/policy/entities/ml-exclusions/v1"], 
            ["deleteMLExclusionsV1", "DELETE", "/policy/entities/ml-exclusions/v1?ids={}"], 
            ["performPreventionPoliciesAction", "POST", "/policy/entities/prevention-actions/v1"], 
            ["setPreventionPoliciesPrecedence", "POST", "/policy/entities/prevention-precedence/v1"], 
            ["getPreventionPolicies", "GET", "/policy/entities/prevention/v1?ids={}"], 
            ["createPreventionPolicies", "POST", "/policy/entities/prevention/v1"], 
            ["updatePreventionPolicies", "PATCH", "/policy/entities/prevention/v1"], 
            ["deletePreventionPolicies", "DELETE", "/policy/entities/prevention/v1?ids={}"], 
            ["performSensorUpdatePoliciesAction", "POST", "/policy/entities/sensor-update-actions/v1"], 
            ["setSensorUpdatePoliciesPrecedence", "POST", "/policy/entities/sensor-update-precedence/v1"], 
            ["getSensorUpdatePolicies", "GET", "/policy/entities/sensor-update/v1?ids={}"], 
            ["createSensorUpdatePolicies", "POST", "/policy/entities/sensor-update/v1"], 
            ["updateSensorUpdatePolicies", "PATCH", "/policy/entities/sensor-update/v1"], 
            ["deleteSensorUpdatePolicies", "DELETE", "/policy/entities/sensor-update/v1?ids={}"], 
            ["getSensorUpdatePoliciesV2", "GET", "/policy/entities/sensor-update/v2?ids={}"], 
            ["createSensorUpdatePoliciesV2", "POST", "/policy/entities/sensor-update/v2"], 
            ["updateSensorUpdatePoliciesV2", "PATCH", "/policy/entities/sensor-update/v2"], 
            ["getSensorVisibilityExclusionsV1", "GET", "/policy/entities/sv-exclusions/v1?ids={}"], 
            ["createSVExclusionsV1", "POST", "/policy/entities/sv-exclusions/v1"], 
            ["updateSensorVisibilityExclusionsV1", "PATCH", "/policy/entities/sv-exclusions/v1"], 
            ["deleteSensorVisibilityExclusionsV1", "DELETE", "/policy/entities/sv-exclusions/v1?ids={}"], 
            ["queryDeviceControlPolicyMembers", "GET", "/policy/queries/device-control-members/v1"], 
            ["queryDeviceControlPolicies", "GET", "/policy/queries/device-control/v1"], 
            ["queryFirewallPolicyMembers", "GET", "/policy/queries/firewall-members/v1"], 
            ["queryFirewallPolicies", "GET", "/policy/queries/firewall/v1"], 
            ["queryIOAExclusionsV1", "GET", "/policy/queries/ioa-exclusions/v1"], 
            ["queryMLExclusionsV1", "GET", "/policy/queries/ml-exclusions/v1"], 
            ["queryPreventionPolicyMembers", "GET", "/policy/queries/prevention-members/v1"], 
            ["queryPreventionPolicies", "GET", "/policy/queries/prevention/v1"], 
            ["querySensorUpdatePolicyMembers", "GET", "/policy/queries/sensor-update-members/v1"], 
            ["querySensorUpdatePolicies", "GET", "/policy/queries/sensor-update/v1"], 
            ["querySensorVisibilityExclusionsV1", "GET", "/policy/queries/sv-exclusions/v1"], 
            ["entities.processes", "GET", "/processes/entities/processes/v1?ids={}"], 
            ["RTR-AggregateSessions", "POST", "/real-time-response/aggregates/sessions/GET/v1"], 
            ["BatchActiveResponderCmd", "POST", "/real-time-response/combined/batch-active-responder-command/v1"], 
            ["BatchAdminCmd", "POST", "/real-time-response/combined/batch-admin-command/v1"], 
            ["BatchCmd", "POST", "/real-time-response/combined/batch-command/v1"], 
            ["BatchGetCmdStatus", "GET", "/real-time-response/combined/batch-get-command/v1"], 
            ["BatchGetCmd", "POST", "/real-time-response/combined/batch-get-command/v1"], 
            ["BatchInitSessions", "POST", "/real-time-response/combined/batch-init-session/v1"], 
            ["BatchRefreshSessions", "POST", "/real-time-response/combined/batch-refresh-session/v1"], 
            ["RTR-CheckActiveResponderCommandStatus", "GET", "/real-time-response/entities/active-responder-command/v1"], 
            ["RTR-ExecuteActiveResponderCommand", "POST", "/real-time-response/entities/active-responder-command/v1"], 
            ["RTR-CheckAdminCommandStatus", "GET", "/real-time-response/entities/admin-command/v1"], 
            ["RTR-ExecuteAdminCommand", "POST", "/real-time-response/entities/admin-command/v1"], 
            ["RTR-CheckCommandStatus", "GET", "/real-time-response/entities/command/v1"], 
            ["RTR-ExecuteCommand", "POST", "/real-time-response/entities/command/v1"], 
            ["RTR-GetExtractedFileContents", "GET", "/real-time-response/entities/extracted-file-contents/v1"], 
            ["RTR-ListFiles", "GET", "/real-time-response/entities/file/v1"], 
            ["RTR-DeleteFile", "DELETE", "/real-time-response/entities/file/v1?ids={}"], 
            ["RTR-GetPut-Files", "GET", "/real-time-response/entities/put-files/v1?ids={}"], 
            ["RTR-CreatePut-Files", "POST", "/real-time-response/entities/put-files/v1"], 
            ["RTR-DeletePut-Files", "DELETE", "/real-time-response/entities/put-files/v1?ids={}"], 
            ["RTR-ListQueuedSessions", "POST", "/real-time-response/entities/queued-sessions/GET/v1"], 
            ["RTR-DeleteQueuedSession", "DELETE", "/real-time-response/entities/queued-sessions/command/v1"], 
            ["RTR-PulseSession", "POST", "/real-time-response/entities/refresh-session/v1"], 
            ["RTR-GetScripts", "GET", "/real-time-response/entities/scripts/v1?ids={}"], 
            ["RTR-CreateScripts", "POST", "/real-time-response/entities/scripts/v1"], 
            ["RTR-UpdateScripts", "PATCH", "/real-time-response/entities/scripts/v1"], 
            ["RTR-DeleteScripts", "DELETE", "/real-time-response/entities/scripts/v1?ids={}"], 
            ["RTR-ListSessions", "POST", "/real-time-response/entities/sessions/GET/v1"], 
            ["RTR-InitSession", "POST", "/real-time-response/entities/sessions/v1"], 
            ["RTR-DeleteSession", "DELETE", "/real-time-response/entities/sessions/v1"], 
            ["RTR-ListPut-Files", "GET", "/real-time-response/queries/put-files/v1"], 
            ["RTR-ListScripts", "GET", "/real-time-response/queries/scripts/v1"], 
            ["RTR-ListAllSessions", "GET", "/real-time-response/queries/sessions/v1"], 
            ["GetSampleV2", "GET", "/samples/entities/samples/v2?ids={}"], 
            ["UploadSampleV2", "POST", "/samples/entities/samples/v2"], 
            ["DeleteSampleV2", "DELETE", "/samples/entities/samples/v2?ids={}"], 
            ["GetSampleV3", "GET", "/samples/entities/samples/v3?ids={}"], 
            ["UploadSampleV3", "POST", "/samples/entities/samples/v3"], 
            ["DeleteSampleV3", "DELETE", "/samples/entities/samples/v3?ids={}"], 
            ["QuerySampleV1", "POST", "/samples/queries/samples/GET/v1"], 
            ["GetScansAggregates", "POST", "/scanner/aggregates/scans/GET/v1"], 
            ["GetScans", "GET", "/scanner/entities/scans/v1?ids={}"], 
            ["ScanSamples", "POST", "/scanner/entities/scans/v1"], 
            ["QuerySubmissionsMixin0", "GET", "/scanner/queries/scans/v1"], 
            ["GetCombinedSensorInstallersByQuery", "GET", "/sensors/combined/installers/v1"], 
            ["refreshActiveStreamSession", "POST", "/sensors/entities/datafeed-actions/v1/{}"], 
            ["listAvailableStreamsOAuth2", "GET", "/sensors/entities/datafeed/v2"], 
            ["DownloadSensorInstallerById", "GET", "/sensors/entities/download-installer/v1"], 
            ["GetSensorInstallersEntities", "GET", "/sensors/entities/installers/v1?ids={}"], 
            ["GetSensorInstallersCCIDByQuery", "GET", "/sensors/queries/installers/ccid/v1"], 
            ["GetSensorInstallersByQuery", "GET", "/sensors/queries/installers/v1"], 
            ["GetCSPMPolicy", "GET", "/settings/entities/policy-details/v1?ids={}"], 
            ["GetCSPMPolicySettings", "GET", "/settings/entities/policy/v1"], 
            ["UpdateCSPMPolicySettings", "PATCH", "/settings/entities/policy/v1"], 
            ["GetCSPMScanSchedule", "GET", "/settings/scan-schedule/v1"], 
            ["UpdateCSPMScanSchedule", "POST", "/settings/scan-schedule/v1"], 
            ["getVulnerabilities", "GET", "/spotlight/entities/vulnerabilities/v2?ids={}"], 
            ["queryVulnerabilities", "GET", "/spotlight/queries/vulnerabilities/v1"], 
            ["GetRoles", "GET", "/user-roles/entities/user-roles/v1?ids={}"], 
            ["GrantUserRoleIds", "POST", "/user-roles/entities/user-roles/v1"], 
            ["RevokeUserRoleIds", "DELETE", "/user-roles/entities/user-roles/v1?ids={}"], 
            ["GetAvailableRoleIds", "GET", "/user-roles/queries/user-role-ids-by-cid/v1"], 
            ["GetUserRoleIds", "GET", "/user-roles/queries/user-role-ids-by-user-uuid/v1"], 
            ["RetrieveUser", "GET", "/users/entities/users/v1?ids={}"], 
            ["CreateUser", "POST", "/users/entities/users/v1"], 
            ["UpdateUser", "PATCH", "/users/entities/users/v1"], 
            ["DeleteUser", "DELETE", "/users/entities/users/v1"], 
            ["RetrieveEmailsByCID", "GET", "/users/queries/emails-by-cid/v1"], 
            ["RetrieveUserUUIDsByCID", "GET", "/users/queries/user-uuids-by-cid/v1"], 
            ["RetrieveUserUUID", "GET", "/users/queries/user-uuids-by-email/v1"]
        ]
        
    class Result:
        """ Subclass to handle parsing of result client output. """
        def __init__(self):
            """ Instantiates the subclass and initializes the result object. """
            self.result_obj = {}

        def __call__(self, status_code, headers, body):
            """ Formats values into a properly formatted result object. """
            self.result_obj['status_code'] = status_code
            self.result_obj['headers'] = dict(headers)
            self.result_obj['body'] = body
            
            return self.result_obj

    def authenticate(self):
        """ Generates an authorization token. """
        FULL_URL = self.base_url+'/oauth2/token'
        DATA = {}
        if self.valid_cred_format():
            DATA = {
                'client_id': self.creds['client_id'],
                'client_secret': self.creds['client_secret']
            }
        try:
            response = requests.request("POST", FULL_URL, data=DATA, headers={}, verify=False)
            result = self.Result()(status_code=response.status_code,headers={},body=response.json())["body"]
            self.token = result["access_token"]
            self.token_expiration = result["expires_in"]
            self.token_time = time.time()
            self.authenticated = True
        except Exception:
            self.authenticated = False
        
        return self.authenticated

    def deauthenticate(self):
        """ Revokes the specified authorization token. """
        FULL_URL = self.base_url+'/oauth2/revoke'
        HEADERS = { 'Authorization': 'basic {}'.format(self.token) }
        DATA = { 'token': '{}'.format(self.token) }
        revoked = False
        try:
            requests.request("POST", FULL_URL, data=DATA, headers=HEADERS, verify=False)
            self.authenticated = False
            self.token = False
            revoked = True
        except Exception:
            revoked = False
            
        return revoked

    def command(self, action="", parameters={}, body={}, data={}, ids=False, partition=False, override=False, files=[], content_type=False):
        """ Checks token expiration, renewing when necessary, then performs the request. """
        if self.token_expired():
            self.authenticate()
        if override:
            CMD = [["Manual"] + override.split(",")]
        else:
            CMD = [a for a in self.commands if a[0] == action]
        if CMD:
            FULL_URL = self.base_url+"{}".format(CMD[0][2])
            if ids:
                ID_LIST = str(ids).replace(",","&ids=")
                FULL_URL = FULL_URL.format(ID_LIST)
            if partition:
                FULL_URL = FULL_URL.format(str(partition))
            HEADERS = self.headers()
            if content_type:
                HEADERS["Content-Type"] = str(content_type)
            DATA = data
            BODY = body
            PARAMS = parameters
            FILES = files
            if self.authenticated:
                try:
                    response = requests.request(CMD[0][1].upper(), FULL_URL, json=BODY, data=DATA, params=PARAMS, headers=HEADERS, files=FILES, verify=False)
                    returned = self.Result()(status_code=response.status_code, headers=response.headers, body=response.json())
                except Exception as e:
                    returned = self.Result()(status_code=500, headers={}, body=str(e))
            else:
                returned = self.Result()(status_code=500, headers={}, body={"errors":[{"message":"Failed to issue token."}],"resources":""})
        else:
            returned = self.Result()(status_code=500, headers={}, body={"errors":[{"message":"Invalid API service method."}],"resources":""})

        return returned



################################################################################################################
# CROWDSTRIKE FALCON                                                                                           #
# oAuth2 API - Customer SDK                                                                                    #
#                                                                                                              #
# intel - Falcon X Threat Intelligence API Interface Class                                                     #
################################################################################################################
# Copyright CrowdStrike 2020

# By accessing or using this script, sample code, application programming interface, tools, 
# and/or associated documentation (if any) (collectively, “Tools”), You (i) represent and 
# warrant that You are entering into this Agreement on behalf of a company, organization 
# or another legal entity (“Entity”) that is currently a customer or partner of 
# CrowdStrike, Inc. (“CrowdStrike”), and (ii) have the authority to bind such Entity and 
# such Entity agrees to be bound by this Agreement.

# CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty 
# free and limited license to access and use the Tools solely for Entity’s internal business 
# purposes and in accordance with its obligations under any agreement(s) it may have with 
# CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its licensors retain all 
# right, title and interest in and to the Tools, and all intellectual property rights 
# embodied therein, and that Entity has no right, title or interest therein except for the 
# express licenses granted hereunder and that Entity will treat such Tools as CrowdStrike’s 
# confidential information.

# THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR 
# STATUTORY OR OTHERWISE. CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND 
# ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL CROWDSTRIKE 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

class Intel:
    """ The only requirement to instantiate an instance of this class
        is a valid token provided by the Falcon API SDK OAuth2 class.
    """

    def __init__(self, access_token, base_url='https://api.crowdstrike.com'):
        """ Instantiates the base class, ingests the authorization token, 
            and initializes the headers and base_url global variables. 
        """
        self.headers = { 'Authorization': 'Bearer {}'.format(access_token) }
        self.base_url = base_url

    class Result:
        """ Subclass to handle parsing of result client output. """
        def __init__(self):
            """ Instantiates the subclass and initializes the result object. """
            self.result_obj = {}
            
        def __call__(self, status_code, headers, body):
            """ Formats values into a properly formatted result object. """
            self.result_obj['status_code'] = status_code
            self.result_obj['headers'] = dict(headers)
            self.result_obj['body'] = body
            
            return self.result_obj

    def QueryIntelActorEntities(self, parameters):
        """ Get info about actors that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelActorEntities
        FULL_URL = self.base_url+'/intel/combined/actors/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelIndicatorEntities(self, parameters):
        """ Get info about indicators that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelIndicatorEntities
        FULL_URL = self.base_url+'/intel/combined/indicators/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelReportEntities(self, parameters):
        """ Get info about reports that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelReportEntities
        FULL_URL = self.base_url+'/intel/combined/reports/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelActorEntities(self, parameters):
        """ Retrieve specific actors using their actor IDs. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelActorEntities
        FULL_URL = self.base_url+'/intel/entities/actors/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelIndicatorEntities(self, body):
        """ Retrieve specific indicators using their indicator IDs. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelIndicatorEntities
        FULL_URL = self.base_url+'/intel/entities/indicators/GET/v1'
        HEADERS = self.headers
        BODY = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelReportPDF(self, parameters):#Probably need to not do result.json() here. Check the swagger
        """ Return a Report PDF attachment. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelReportPDF
        FULL_URL = self.base_url+'/intel/entities/report-files/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelReportEntities(self, parameters):
        """ Retrieve specific reports using their report IDs. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelReportEntities
        FULL_URL = self.base_url+'/intel/entities/reports/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelRuleFile(self, parameters):#There is an optional header you can see Accept to choose the result format. See Swagger.
        """ Download earlier rule sets. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelRuleFile
        FULL_URL = self.base_url+'/intel/entities/rules-files/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetLatestIntelRuleFile(self, parameters):#There is an optional header you can see Accept to choose the result format. See Swagger.
        """ Download the latest rule set. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetLatestIntelRuleFile
        FULL_URL = self.base_url+'/intel/entities/rules-latest-files/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetIntelRuleEntities(self, parameters):
        """ Retrieve details for rule sets for the specified ids. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/GetIntelRuleEntities
        FULL_URL = self.base_url+'/intel/entities/rules/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelActorIds(self, parameters):
        """ Get actor IDs that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelActorIds
        FULL_URL = self.base_url+'/intel/queries/actors/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelIndicatorIds(self, parameters):
        """ Get indicators IDs that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelIndicatorIds
        FULL_URL = self.base_url+'/intel/queries/indicators/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelReportIds(self, parameters):
        """ Get report IDs that match provided FQL filters. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelReportIds
        FULL_URL = self.base_url+'/intel/queries/reports/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryIntelRuleIds(self, parameters):
        """ Search for rule IDs that match provided filter criteria. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/intel/QueryIntelReportIds
        FULL_URL = self.base_url+'/intel/queries/rules/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned



################################################################################################################
# CROWDSTRIKE FALCON                                                                                           #
# oAuth2 API - Customer SDK                                                                                    #
#                                                                                                              #
# prevention_policy - Falcon X Prevention Policy API Interface Class                                           #
################################################################################################################
# Copyright CrowdStrike 2020

# By accessing or using this script, sample code, application programming interface, tools, 
# and/or associated documentation (if any) (collectively, “Tools”), You (i) represent and 
# warrant that You are entering into this Agreement on behalf of a company, organization 
# or another legal entity (“Entity”) that is currently a customer or partner of 
# CrowdStrike, Inc. (“CrowdStrike”), and (ii) have the authority to bind such Entity and 
# such Entity agrees to be bound by this Agreement.

# CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty 
# free and limited license to access and use the Tools solely for Entity’s internal business 
# purposes and in accordance with its obligations under any agreement(s) it may have with 
# CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its licensors retain all 
# right, title and interest in and to the Tools, and all intellectual property rights 
# embodied therein, and that Entity has no right, title or interest therein except for the 
# express licenses granted hereunder and that Entity will treat such Tools as CrowdStrike’s 
# confidential information.

# THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR 
# STATUTORY OR OTHERWISE. CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND 
# ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL CROWDSTRIKE 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

class Prevention_Policy:
    """ The only requirement to instantiate an instance of this class
        is a valid token provided by the Falcon API SDK OAuth2 class.
    """

    def __init__(self, access_token, base_url='https://api.crowdstrike.com'):
        """ Instantiates the base class, ingests the authorization token, 
            and initializes the headers and base_url global variables. 
        """
        self.headers = { 'Authorization': 'Bearer {}'.format(access_token) }
        self.base_url = base_url

    class Result:
        """ Subclass to handle parsing of result client output. """
        def __init__(self):
            """ Instantiates the subclass and initializes the result object. """
            self.result_obj = {}
            
        def __call__(self, status_code, headers, body):
            """ Formats values into a properly formatted result object. """
            self.result_obj['status_code'] = status_code
            self.result_obj['headers'] = dict(headers)
            self.result_obj['body'] = body
            
            return self.result_obj

    def queryCombinedPreventionPolicyMembers(self, parameters):
        """ Search for members of a Prevention Policy in your environment by providing an FQL filter 
            and paging details. Returns a set of host details which match the filter criteria.
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/queryCombinedPreventionPolicyMembers
        FULL_URL = self.base_url+'/policy/combined/prevention-members/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def queryCombinedPreventionPolicies(self, parameters):
        """ Search for Prevention Policies in your environment by providing an FQL filter and 
            paging details. Returns a set of Prevention Policies which match the filter criteria. 
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/queryCombinedPreventionPolicies
        FULL_URL = self.base_url+'/policy/combined/prevention/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def performPreventionPoliciesAction(self, parameters, body):
        """ Perform the specified action on the Prevention Policies specified in the request. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/performPreventionPoliciesAction
        FULL_URL = self.base_url+'/policy/entities/prevention-actions/v1'
        HEADERS = self.headers
        PARAMS = parameters
        BODY = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def setPreventionPoliciesPrecedence(self, body):
        """ Sets the precedence of Prevention Policies based on the order of IDs specified in the request.
            The first ID specified will have the highest precedence and the last ID specified will have the lowest.
            You must specify all non-Default Policies for a platform when updating precedence.
        """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/setPreventionPoliciesPrecedence
        FULL_URL = self.base_url+'/policy/entities/prevention-precedence/v1'
        HEADERS = self.headers
        BODY = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def getPreventionPolicies(self, parameters):
        """ Retrieve a set of Prevention Policies by specifying their IDs. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/getPreventionPolicies
        FULL_URL = self.base_url+'/policy/entities/prevention/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def createPreventionPolicies(self, body):
        """ Create Prevention Policies by specifying details about the policy to create. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/createPreventionPolicies
        FULL_URL = self.base_url+'/policy/entities/prevention/v1'
        HEADERS = self.headers
        BODY = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def deletePreventionPolicies(self, parameters):
        """ Delete a set of Prevention Policies by specifying their IDs. """
        # [DELETE] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/deletePreventionPolicies
        FULL_URL = self.base_url+'/policy/entities/prevention/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("DELETE", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def updatePreventionPolicies(self, body):
        """ Update Prevention Policies by specifying the ID of the policy and details to update. """
        # [PATCH] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/updatePreventionPolicies
        FULL_URL = self.base_url+'/policy/entities/prevention/v1'
        HEADERS = self.headers
        BODY = body
        result = self.Result()
        try:
            response = requests.request("PATCH", FULL_URL, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def queryPreventionPolicyMembers(self, parameters):
        """ Search for members of a Prevention Policy in your environment by providing an FQL filter 
            and paging details. Returns a set of Agent IDs which match the filter criteria.
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/queryPreventionPolicyMembers
        FULL_URL = self.base_url+'/policy/queries/prevention-members/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def queryPreventionPolicies(self, parameters):
        """ Search for Prevention Policies in your environment by providing an FQL filter 
            and paging details. Returns a set of Prevention Policy IDs which match the filter criteria. 
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/prevention-policies/queryPreventionPolicies
        FULL_URL = self.base_url+'/policy/queries/prevention/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned



################################################################################################################
# CROWDSTRIKE FALCON                                                                                           #
# oAuth2 API - Customer SDK                                                                                    #
#                                                                                                              #
# cloud_connect_aws - Falcon X Discover API Interface Class                                                    #
################################################################################################################
# Copyright CrowdStrike 2020

# By accessing or using this script, sample code, application programming interface, tools, 
# and/or associated documentation (if any) (collectively, “Tools”), You (i) represent and 
# warrant that You are entering into this Agreement on behalf of a company, organization 
# or another legal entity (“Entity”) that is currently a customer or partner of 
# CrowdStrike, Inc. (“CrowdStrike”), and (ii) have the authority to bind such Entity and 
# such Entity agrees to be bound by this Agreement.

# CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty 
# free and limited license to access and use the Tools solely for Entity’s internal business 
# purposes and in accordance with its obligations under any agreement(s) it may have with 
# CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its licensors retain all 
# right, title and interest in and to the Tools, and all intellectual property rights 
# embodied therein, and that Entity has no right, title or interest therein except for the 
# express licenses granted hereunder and that Entity will treat such Tools as CrowdStrike’s 
# confidential information.

# THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR 
# STATUTORY OR OTHERWISE. CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND 
# ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL CROWDSTRIKE 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

class Cloud_Connect_AWS:
    """ The only requirement to instantiate an instance of this class
        is a valid token provided by the Falcon API SDK OAuth2 class.
    """

    def __init__(self, access_token, base_url="https://api.crowdstrike.com"):
        """ Instantiates the base class, ingests the authorization token, 
            and initializes the headers and base_url global variables. 
        """
        self.headers = {'Authorization': 'Bearer {}'.format(access_token)}
        self.base_url = base_url

    class Result:
        """ Subclass to handle parsing of result client output. """
        def __init__(self):
            """ Instantiates the subclass and initializes the result object. """
            self.result_obj = {}
            
        def __call__(self, status_code, headers, body):
            """ Formats values into a properly formatted result object. """
            self.result_obj['status_code'] = status_code
            self.result_obj['headers'] = dict(headers)
            self.result_obj['body'] = body
            
            return self.result_obj

    def QueryAWSAccounts(self, parameters):
        """ Search for provisioned AWS Accounts by providing an FQL filter and paging details. 
            Returns a set of AWS accounts which match the filter criteria. 
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/QueryAWSAccounts 
        FULL_URL = self.base_url+'/cloud-connect-aws/combined/accounts/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def GetAWSSettings(self):
        """ Retrieve a set of Global Settings which are applicable to all provisioned AWS accounts. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/GetAWSSettings
        FULL_URL = self.base_url+'/cloud-connect-aws/combined/settings/v1'
        HEADERS = self.headers
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
            
        return returned

    def GetAWSAccounts(self, parameters, ids):
        """ Retrieve a set of AWS Accounts by specifying their IDs."""
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/GetAWSAccounts
        ID_LIST = str(ids).replace(",","&ids=")
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/accounts/v1?ids={}'.format(ID_LIST)
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))

        return returned
        
    def ProvisionAWSAccounts(self, parameters, body):
        """ Provision AWS Accounts by specifying details about the accounts to provision. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/ProvisionAWSAccounts
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/accounts/v1'
        PARAMS=parameters
        BODY=body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=BODY, headers=self.headers, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))

        return returned
        
    def DeleteAWSAccounts(self, parameters, ids):
        """ Delete a set of AWS Accounts by specifying their IDs. """
        # [DELETE] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/DeleteAWSAccounts
        ID_LIST = str(ids).replace(",","&ids=")
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/accounts/v1?ids={}'.format(ID_LIST)
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("DELETE", FULL_URL, params=PARAMS, headers=self.headers, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def UpdateAWSAccounts(self, body):
        """ Update AWS Accounts by specifying the ID of the account and details to update. """
        # [PATCH] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/UpdateAWSAccounts
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/accounts/v1'
        BODY=body
        result = self.Result()
        try:
            response = requests.request("PATCH", FULL_URL, json=BODY, headers=self.headers, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def CreateOrUpdateAWSSettings(self, body):
        """ Create or update Global Settings which are applicable to all provisioned AWS accounts. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/CreateOrUpdateAWSSettings
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/settings/v1'
        HEADERS = self.headers
        BODY = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=BODY, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
            
        return returned

    def VerifyAWSAccountAccess(self, parameters, body, ids):
        """ Performs an Access Verification check on the specified AWS Account IDs. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/VerifyAWSAccountAccess
        ID_LIST = str(ids).replace(",","&ids=")
        FULL_URL = self.base_url+'/cloud-connect-aws/entities/verify-account-access/v1?ids={}'.format(ID_LIST)
        HEADERS = self.headers
        PARAMS = parameters
        BODY = body  #payload does not appear to be required
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=BODY, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def QueryAWSAccountsForIDs(self, parameters):
        """ Search for provisioned AWS Accounts by providing an FQL filter and paging details. 
            Returns a set of AWS account IDs which match the filter criteria. 
        """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/cloud-connect-aws/QueryAWSAccountsForIDs
        FULL_URL = self.base_url+'/cloud-connect-aws/queries/accounts/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
            
        return returned



import os
import setuptools
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

here = os.path.abspath(os.path.dirname(__file__))

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="csfalcon", # Replace with your own username
    version="0.1.1",
    author="CrowdStrike",
    maintainer="Joshua Hiller",
    description="The CrowdStrike Falcon API SDK for Python 3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/CrowdStrike/csfalcon",
    packages=setuptools.find_packages('csfalcon'),
    package_dir={'': 'csfalcon'},
    py_modules=[splitext(basename(path))[0] for path in glob('csfalcon/*.py')],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
	"requests",
	"urllib3"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: The Unlicense (Unlicense)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)




################################################################################################################
# CROWDSTRIKE FALCON                                                                                           #
# oAuth2 API - Customer SDK                                                                                    #
#                                                                                                              #
# real_time_response - Falcon X Real Time Response API Interface Class                                         #
################################################################################################################
# Copyright CrowdStrike 2020

# By accessing or using this script, sample code, application programming interface, tools, 
# and/or associated documentation (if any) (collectively, “Tools”), You (i) represent and 
# warrant that You are entering into this Agreement on behalf of a company, organization 
# or another legal entity (“Entity”) that is currently a customer or partner of 
# CrowdStrike, Inc. (“CrowdStrike”), and (ii) have the authority to bind such Entity and 
# such Entity agrees to be bound by this Agreement.

# CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty 
# free and limited license to access and use the Tools solely for Entity’s internal business 
# purposes and in accordance with its obligations under any agreement(s) it may have with 
# CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its licensors retain all 
# right, title and interest in and to the Tools, and all intellectual property rights 
# embodied therein, and that Entity has no right, title or interest therein except for the 
# express licenses granted hereunder and that Entity will treat such Tools as CrowdStrike’s 
# confidential information.

# THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR 
# STATUTORY OR OTHERWISE. CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND 
# ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL CROWDSTRIKE 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import requests
import json
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)

class Real_Time_Response:
    """ The only requirement to instantiate an instance of this class
        is a valid token provided by the Falcon API SDK OAuth2 class.
    """

    def __init__(self, access_token, base_url='https://api.crowdstrike.com'):
        """ Instantiates the base class, ingests the authorization token, 
            and initializes the headers and base_url global variables. 
        """
        self.headers = { 'Authorization': 'Bearer {}'.format(access_token) }
        self.base_url = base_url

    class Result:
        """ Subclass to handle parsing of result client output. """
        def __init__(self):
            """ Instantiates the subclass and initializes the result object. """
            self.result_obj = {}
            
        def __call__(self, status_code, headers, body):
            """ Formats values into a properly formatted result object. """
            self.result_obj['status_code'] = status_code
            self.result_obj['headers'] = dict(headers)
            self.result_obj['body'] = body
            
            return self.result_obj

    def RTR_AggregateSessions(self, body):
        """ Get aggregates on session data. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_AggregateSessions
        FULL_URL = self.base_url+'/real-time-response/aggregates/sessions/GET/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchActiveResponderCmd(self, parameters, body):
        """ Batch executes a RTR active-responder command across the hosts mapped to the given batch ID. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchActiveResponderCmd
        FULL_URL = self.base_url+'/real-time-response/combined/batch-active-responder-command/v1'
        HEADERS = self.headers
        DATA = body
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchCmd(self, parameters, body):
        """ Batch executes a RTR read-only command across the hosts mapped to the given batch ID. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchActiveResponderCmd
        FULL_URL = self.base_url+'/real-time-response/combined/batch-command/v1'
        HEADERS = self.headers
        DATA = body
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchGetCmdStatus(self, parameters):
        """ Retrieves the status of the specified batch get command. Will return successful files when they are finished processing. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchGetCmdStatus
        FULL_URL = self.base_url+'/real-time-response/combined/batch-get-command/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchGetCmd(self, parameters, body):
        """ Batch executes `get` command across hosts to retrieve files. 
            After this call is made `/real-time-response/combined/get-command-status/v1` is used to query for the results.
        """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchActiveResponderCmd
        FULL_URL = self.base_url+'/real-time-response/combined/batch-get-command/v1'
        HEADERS = self.headers
        DATA = body
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchInitSessions(self, parameters, body):
        """ Batch initialize a RTR session on multiple hosts. Before any RTR commands can be used, an active session is needed on the host. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchInitSessions
        FULL_URL = self.base_url+'/real-time-response/combined/batch-init-session/v1'
        HEADERS = self.headers
        DATA = body
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def BatchRefreshSessions(self, parameters, body):
        """ Batch refresh a RTR session on multiple hosts. RTR sessions will expire after 10 minutes unless refreshed. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchRefreshSessions
        FULL_URL = self.base_url+'/real-time-response/combined/batch-refresh-session/v1'
        HEADERS = self.headers
        DATA = body
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, params=PARAMS, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_CheckActiveResponderCommandStatus(self, parameters):
        """ Get status of an executed active-responder command on a single host. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_CheckActiveResponderCommandStatus
        FULL_URL = self.base_url+'/real-time-response/entities/active-responder-command/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_ExecuteActiveResponderCommand(self, body):
        """ Execute an active responder command on a single host. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/BatchRefreshSessions
        FULL_URL = self.base_url+'/real-time-response/entities/active-responder-command/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_CheckCommandStatus(self, parameters):
        """ Get status of an executed command on a single host. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_CheckCommandStatus
        FULL_URL = self.base_url+'/real-time-response/entities/command/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_ExecuteCommand(self, body):
        """ Execute a command on a single host. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_ExecuteCommand
        FULL_URL = self.base_url+'/real-time-response/entities/command/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_GetExtractedFileContents(self, parameters):
        """ Get RTR extracted file contents for specified session and sha256. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_GetExtractedFileContents
        FULL_URL = self.base_url+'/real-time-response/entities/extracted-file-contents/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_ListFiles(self, parameters):
        """ Get a list of files for the specified RTR session. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_ListFiles
        FULL_URL = self.base_url+'/real-time-response/entities/file/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_DeleteFile(self, parameters):
        """ Delete a RTR session file. """
        # [DELETE] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_DeleteFile
        FULL_URL = self.base_url+'/real-time-response/entities/file/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("DELETE", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_PulseSession(self, body):
        """ Refresh a session timeout on a single host. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_PulseSession
        FULL_URL = self.base_url+'/real-time-response/entities/refresh-session/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_ListSessions(self, body):
        """ Get session metadata by session id. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_ListSessions
        FULL_URL = self.base_url+'/real-time-response/entities/sessions/GET/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def InitSessionMixin0(self, body):
        """ Initialize a new session with the RTR cloud. """
        # [POST] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/InitSessionMixin0
        FULL_URL = self.base_url+'/real-time-response/entities/sessions/v1'
        HEADERS = self.headers
        DATA = body
        result = self.Result()
        try:
            response = requests.request("POST", FULL_URL, json=DATA, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_DeleteSession(self, parameters):
        """ Delete a session. """
        # [DELETE] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_DeleteSession
        FULL_URL = self.base_url+'/real-time-response/entities/sessions/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("DELETE", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned

    def RTR_ListAllSessions(self, parameters):
        """ Get a list of session_ids. """
        # [GET] https://assets.falcon.crowdstrike.com/support/api/swagger.html#/real-time-response/RTR_ListAllSessions
        FULL_URL = self.base_url+'/real-time-response/queries/sessions/v1'
        HEADERS = self.headers
        PARAMS = parameters
        result = self.Result()
        try:
            response = requests.request("GET", FULL_URL, params=PARAMS, headers=HEADERS, verify=False)
            returned = result(response.status_code, response.headers, response.json())
        except Exception as e:
            returned = result(500, {}, str(e))
        
        return returned
